# YASARA MOVIE
# TOPIC:    6. Multimedia
# TITLE:    6.3. Buckynut Island
# REQUIRES: Model 5.10.1
# AUTHOR:   Elmar Krieger
# LICENSE:  GNU GPL
# DESCRIPTION: A video game written in Yanaconda to demonstrate the use of solids. Help Yami collect the Buckminster nuts. (Current highscore is 215 points by Bas Vroling)

# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)

# Initialize
Start:
Clear
UserInput Off
CoordSys Left
Projection perspective
Console Off
HUD Off
HideMessage

# Set basic graphics styles
StickRadius 50
BallStickRadius 50,50
ColorBonds Grey
Fog 60
AtomSize 100
# Only consider shadows with Vulkan rendering
shadow=0
renderer = Renderer
if renderer=='Vulkan'
  shadow=50
LightSource 50,50,Ambience=0,Shadow=(shadow)

# Create the background
LoadPNG skysea
ShowImage skysea,0,0,1024,600
ColorFog a8b6d6
Fog 100

# Create the island
islandheight=20.
islandradius=200
ShowCone BottomRadius=(islandradius*1.5),TopRadius=(0.998*islandradius),Height=(islandheight),Edges=100,
         BottomCol=None,TopCol=None,SideCol=c5a885
MoveObj Cone,Y=(-islandheight/2)
# Include the sand
LoadPNG sand
MakeImageObj Sand,sand,(islandradius*2),(islandradius*2)
RotateObj Sand,X=90

# Add the water
LoadPNG waves
MakeImageObj Waves,waves,Width=1000,Height=1000
RotateObj Waves,X=90
MoveObj Waves,Y=-(islandheight/2)
AutoRotateObj Waves,Y=0.03
wavestep=0

# Add the wooden board with instructions
boarddepth=4
boardlen=26
boardheight=30
obj = ShowCone BottomRadius=(boarddepth/2),Height=(boardheight),Edges=12,
               BottomCol=None,SideCol=746b6c
MoveMesh (obj),Y=-((boardlen+boardheight)/2)
ShowBox Width=(boardlen),Height=(boardlen),Depth=(boarddepth),
        LeftCol=746b6c,FrontCol=None,BackCol=746b6c
LoadPNG board
MakeImageObj Board,board,(boardlen),(boardlen)
MoveObj Board,Z=-(boarddepth/2)
NameObj (obj)-(obj+2),Board
RotateObj Board,Z=-13
MoveObj Board,X=200,Y=30,Z=-140

# Create Yami's floating matrass
matsegments=5
matsegradius=4.0
matlength=60
matcol='Green'
for i=0 to matsegments-1
  obj = ShowCone BottomRadius=(matsegradius),Height=(matlength),Edges=12,
                 BottomCol=None,SideCol=(matcol)
  dx=(1.50*matsegradius*((-0.5*(matsegments-1))+i))
  MoveMesh (obj),X=(dx)
  RotateObj (obj),X=90
  NameObj (obj),Matrass
  for j=-1 to 1 step 2
    obj = ShowSphere Radius=(matsegradius),Color=(matcol),Level=2
    MoveMesh (obj),X=(dx),Z=(0.5*j*matlength)
    NameObj (obj),Matrass
obj = ShowSphere Radius=(0.75*matsegradius*matsegments),Color=(matcol),Level=2,ScaleY=0.5,ScaleZ=0.5
MoveMesh (obj),Y=(1.50*matsegradius),Z=(0.5*matlength)
NameObj (obj),Matrass
matshift=225
MoveObj Matrass,X=(matshift),Y=-10,Z=(-matshift)

# Define the look of a palm tree
stamheight=12.
stamradius=3.
stamsegments=8
crown=stamsegments
leafs=5
treeheight=stamheight*stamsegments
treeobjects=stamsegments+leafs

# Define the positions of the palm trees on the island
treeposx=40, -110,  70,  -90, -180, 130, 100
treeposz=40,   80, -60, -120,    0, 130, 0
trees=count treeposx

# Grow the palm trees
for i=1 to trees
  # Create the palm stam  
  for j=0 to stamsegments-1
    obj = ShowCone BottomRadius=(stamradius),TopRadius=(stamradius*2.0),Height=(stamheight),Edges=12,
                   BottomCol=None,TopCol=3a200a,SideCol=986c34
    MoveMesh (obj),Y=(stamheight*j)
    NameObj (obj),Tree(i)(j)
  # Create the palm crown
  for j=1 to leafs
    obj = ShowSphere Radius=(stamradius*2.0),Color=3aae35,Level=2,ScaleY=0.8,ScaleZ=5
    RotateObj (obj),X=8,Y=(j*360/leafs)
    MoveMesh (obj),Y=(treeheight-0.5*stamheight)
    NameObj (obj),Tree(i)(crown)
  # Move the tree to the final location
  MoveObj Tree(i)?,X=(treeposx(i)),Z=(treeposz(i))

# Get Yami
LoadYOb yami
MoveObj yami,Y=22.6

# Place the scene
cameraheight=30
Move Y=(-cameraheight),Z=80
cenx,ceny,cenz=PosObj yami
Move X=-200,Z=220

# The Intro
# =========
# Place Yami on the matrass
MoveObj Yami,X=(matshift),Y=-25,Z=(-matshift)
MoveAtom Obj Yami,Y=10
RotateObj Yami,Y=90
RotateObj Yami,X=70
AutoRotateObj Yami Matrass,Y=0.2
treestep=0
Wait 1
# Take snapshot here, Eliza!

# Animate the scene
done=0
while not done or treestep%180
  if not done
    ShowMessage "Welcome to the first YASARA macro game! LeftButton for instructions, RightButton to start..."
  # Animate the waves
  MoveObj Yami Matrass Waves,Y=((sin wavestep)*0.05)
  wavestep=wavestep+1
  # Animate the palm trees
  for i=1 to stamsegments
    RotateObj Tree?(i),Z=(cos treestep*i*0.02)
  treestep=treestep+1
  if LeftButton and not done
    HideMessage
    # Show instructions
    AutoPos Z=142,Steps=100
    Wait ContinueButton
    AutoPos Z=320.728,Steps=100
  if RightButton
    HideMessage
    done=1
  Wait 1  

# Zoom in
MoveAtom Obj Yami,Y=-10
AutoRotateObj Yami,Y=0
AutoPosObj Yami,(cenx),(ceny),(cenz),Steps=100,Wait=No
AutoOriObj Yami,0,-90,0,Steps=100,Wait=No
AutoMove X=2,Z=-2.2
Wait 100
AutoMove X=0,Z=0  

# Load the nut
LoadYOb nut
ColorAtom Obj Nut,Blue,Cyan
MoveAtom Obj Nut,X=(stamradius*10),Y=(treeheight-stamheight*1.3)
nutontreey=-cameraheight
nutgroundy=nutontreey-treeheight
PosObj Nut,Y=(nutgroundy)

if !shadow
  # No automatic shadows, create a filled circle below Yami to indicate his position
  shadowobj = ShowCone BottomRadius=10,Height=0,Edges=12,BottomCol=dfb0a3
  shadowy=-22.3

# The Game
# ========
# Yami's position, speed in Angstrom/screen update, and direction of movement
yamiposx=0.
yamiposz=0.
speed=0.5e0
jumpstep=0
jumpsteps=100
dir=270
time=9000
score=0
LabelPar Font=Arial
nuttree=0
nutfalling=1
# Main loop
while time>0
  # Yami is always at the center cenx/cenz. The island position on screen (islepos)
  # is thus the center minus Yami's position on the island (yamipos).
  isleposx=cenx-yamiposx
  isleposz=cenz-yamiposz
  # Animate the waves
  #PosObj Waves,Y=((cos wavestep)*islandheight/4-islandheight/2-cameraheight)
  MoveObj Matrass Waves,Y=((sin wavestep)*0.05)
  PosObj Matrass,X=(isleposx+cos (-0.05*wavestep)*matshift*1.3),
                 Z=(isleposz+sin (-0.05*wavestep)*matshift*1.3)
  wavestep=wavestep+1
  # Handle the bucky nut
  if nutfalling
    # It's currently falling down
    MoveObj Nut,Y=(-0.1*nutfalling)
    nutfalling=nutfalling+1
    x,y=PosObj Nut
    if y<nutgroundy
      # Nut hit the ground, choose a new position
      nuttree=1+trunc (rnd trees)
      PosObj Nut,X=(isleposx+treeposx(nuttree)),Y=(nutontreey),Z=(isleposz+treeposz(nuttree))
      OriObj Nut,Beta=(-90+trunc (rnd leafs)*360/leafs)
      MarkAtom 1013
      nutfalling=0
  # Move Yami
  stepx= speed*cos dir
  stepz=-speed*sin dir
  yamiposx=yamiposx+stepx
  yamiposz=yamiposz+stepz
  Move X=(-stepx),Z=(-stepz)
  # Place Yami
  height=sin (180.*jumpstep/jumpsteps)*speed*18
  PosObj yami,(cenx),(ceny+height),(cenz)
  OriObj Yami,0,0,0
  RotateObj Yami,Z=(sin (360.*jumpstep/jumpsteps)*-30)
  RotateObj Yami,Y=(dir)
  # See if Yami is on the island
  onisland=(sqr yamiposx+sqr yamiposz<sqr (islandradius+25))
  if !shadow
    # No atomatic shadows, show Yami's pseudo-shadow circle only on the island
    PosObj (shadowobj),(cenx),(ceny+shadowy),(cenz)
    # SwitchObj expects strings Off,On,OffOn - these are selected by number (onisland+1)
    SwitchObj (shadowobj),(onisland+1)
  # Jump along
  jumpstep=jumpstep+1
  if jumpstep>=jumpsteps
    jumpstep=0
    # See if Yami fell into the water
    if not onisland
      # Oh noo...
      if !shadow
        DelObj (shadowobj)
      AutoMoveObj !Yami,X=(-yamiposx*0.4/islandradius),Z=(-yamiposz*0.4/islandradius)
      AutoMoveObj Yami,Y=-0.4
      AutoRotateObj Yami,2,3,4
      Wait 200
      AutoMoveObj all,0,0,0
      break
  # Rotate Yami's antennas
  Dihedral N Res Lys,CA Res Lys,CB Res Lys,CG Res Lys Obj Yami,bound=Yes,set=(-180+360*jumpstep/jumpsteps)
  # Steer Yami
  if MiddleButton or (LeftButton and RightButton)
    # Accelerate
    speed=speed+0.01
    if speed>2
      speed=2.
  if LeftButton
    # Turn left
    dir=dir-2
  if RightButton
    # Turn right
    dir=dir+2
  dir=(dir+360)%360
  # Detect collision with trees
  for i=1 to trees
    dx=yamiposx-treeposx(i)
    dz=yamiposz-treeposz(i)
    if dx*dx+dz*dz<100
      # Bumped into tree
      swingsteps=30
      cosdir=cos dir
      sindir=sin dir
      # Determine the axis to swing about and the strength of the bump
      # (i.e. the rotation angle of the tree segments)
      if abs cosdir>0.707
        swingaxis='Z'
        rotangle=speed*-0.08*sign cosdir
      else
        swingaxis='X'
        rotangle=speed*-0.08*sign sindir
      rotangle=rotangle*sin (180*jumpstep/jumpsteps)
      if i==nuttree
        # Yami hit the tree with the nut
        if abs rotangle>0.12
          # Bump was strong enough
          nutfalling=1
          points=6+(abs rotangle-0.12)*100
          score=score+points
          LabelAll '(points) Points!!',Height=1.6,Color=Yellow,X=0,Y=0
        else
          # Bump was not strong enough
          LabelAll 'Too weak..',Height=1,Color=Black,X=0,Y=0
      # Tree swings back and forth
      for j=1 to 2
        if j==1
          # Yami falls
          AutoOriObj yami,Beta=(dir),Gamma=90,Steps=(swingsteps),Wait=No
          AutoPosObj yami,(cenx),(ceny-20),(cenz),Steps=(swingsteps),Wait=No
        else
          # And jumps up
          dir=(dir+180)%360
          AutoOriObj yami,Beta=(dir),Gamma=0,Steps=(swingsteps),Wait=No
          AutoPosObj yami,(cenx),(ceny),(cenz),Steps=(swingsteps),Wait=No
        for k=1 to swingsteps
          for l=1 to stamsegments
            RotateObj Tree(i)(l),(swingaxis)=(rotangle*l)
          if i==nuttree and not nutfalling
            RotateObj Nut,(swingaxis)=(rotangle*stamsegments)
          yamiposx=yamiposx-stepx
          yamiposz=yamiposz-stepz
          Move X=(stepx),Z=(stepz)
          PosObj yami,X=(cenx),Z=(cenz)
          if !shadow
            PosObj (shadowobj),(cenx),(ceny+shadowy),(cenz)
          if nutfalling
            MoveObj Nut,Y=(-0.1*nutfalling)
            nutfalling=nutfalling+1
          # Draw the scene
          Wait 1
        rotangle=-rotangle
        speed=0.5e0
        stepx= speed*cos dir
        stepz=-speed*sin dir
      jumpstep=0
      break
  # Show Time & Score
  UnlabelAll
  LabelAll 'Score (score)',Height=0.5,Color=8080ff,X=-4,Y=3.9
  LabelAll 'Time (time/100)',Height=0.5,Color=8080ff,X=4,Y=3.9
  time=time-1
  # Draw the scene
  Wait 1
# Finished
AutoRotateObj all,0,0,0
if Model and not Movie
  ShowMessage 'Press <Ctrl>+<M> to restart...'
LabelAll 'Your score is',Height=1,Color=Yellow,Y=2
LabelAll '(score)',Height=3,Color=Yellow,Y=-1
AutoOri -55,0,0,Steps=200,Wait=No
x1,y1,z1=PosObj 1
x2,y2,z2=PosAll
AutoPos (x2-x1-33),(y2-y1+16),(z2-z1+84),Steps=200
AutoMove Z=0.3
# Commented out, because it looks too much like a Tsunami ;-(
#RotateObj Waves,Y=10
#MoveObj Waves,Y=-20,Z=20
Wait 500
#AutoMoveObj Waves,Y=0.15
Wait 1500
AutoMove Z=0
Go Start


